/****************************************************************************
 *  Copyright (C) 2017 Kron Technologies Inc <http://www.krontech.ca>.      *
 *                                                                          *
 *  This program is free software: you can redistribute it and/or modify    *
 *  it under the terms of the GNU General Public License as published by    *
 *  the Free Software Foundation, either version 3 of the License, or       *
 *  (at your option) any later version.                                     *
 *                                                                          *
 *  This program is distributed in the hope that it will be useful,         *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License       *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.   *
 ****************************************************************************/

#include "lux1310.h"

static const uint8_t lux1310_wt_sram80_data[] = {
    0x81, 0xFF, 0x27, 0xFA, 0x21, 0xA0, 0x7F, 0xCB, 0xFE, 0x8C, 0x78, 0x1F, 0xF2, 0xFF, 0xA3,
    0x1E, 0x07, 0xFC, 0xBF, 0xE8, 0xC7, 0x81, 0xFF, 0x2F, 0xFA, 0x31, 0xE0, 0x7F, 0xCB, 0xFE,
    0x8C, 0x78, 0x1F, 0xF2, 0xFF, 0xA3, 0x1E, 0x07, 0xFC, 0xBF, 0xE8, 0xC7, 0x81, 0xFF, 0x2F,
    0xFA, 0x31, 0xE0, 0x7F, 0xCB, 0xFE, 0x8C, 0x78, 0x1F, 0xF2, 0xFF, 0xA3, 0x1E, 0x07, 0xFC,
    0xBF, 0xE8, 0xC7, 0x81, 0xFF, 0x2F, 0xFA, 0x31, 0xE0, 0x7F, 0xCB, 0xFE, 0x8C, 0x78, 0x1F,
    0xF2, 0xFF, 0xA3, 0x1E, 0x07, 0xFC, 0xBF, 0xE8, 0xC7, 0x81, 0xFF, 0x2F, 0xFA, 0x31, 0xE0,
    0x7F, 0xCB, 0xFE, 0x8C, 0x78, 0x1F, 0xF2, 0xFF, 0xA3, 0x1E, 0x07, 0xFC, 0xBF, 0xE8, 0xC7,
    0x81, 0xFF, 0x2F, 0xFA, 0x31, 0xE0, 0x7F, 0xCB, 0xFE, 0x8C, 0x78, 0x1F, 0xF2, 0xFF, 0xA3,
    0x1E, 0x07, 0xFC, 0xBF, 0xE8, 0xC7, 0x81, 0xFF, 0x2F, 0xFA, 0x31, 0xE0, 0x7F, 0xCB, 0xFE,
    0x84, 0x78, 0x1F, 0xF2, 0xFF, 0xA1, 0x1E, 0x07, 0xFC, 0xBF, 0xE8, 0x47, 0x81, 0xFF, 0x2F,
    0xFA, 0x11, 0xE0, 0x7F, 0xCB, 0xFE, 0x84, 0x78, 0x1F, 0xF2, 0xFF, 0xA1, 0x1E, 0x07, 0xFC,
    0xBF, 0xE8, 0x47, 0x81, 0xFF, 0x2F, 0xFA, 0x11, 0xE0, 0x7F, 0xCB, 0xFE, 0x84, 0x78, 0x1F,
    0xF2, 0xFF, 0xA1, 0x1E, 0x07, 0xFC, 0xBF, 0xE8, 0x47, 0x81, 0xFF, 0x2F, 0xFA, 0x11, 0xE0,
    0x7F, 0xCB, 0xFE, 0x84, 0x78, 0x1F, 0xF2, 0xFF, 0xA1, 0x1E, 0x07, 0xFC, 0xBF, 0xE8, 0x47,
    0x80, 0xFF, 0x2F, 0xFA, 0x11, 0xE0, 0x3F, 0xC3, 0xFE, 0x94, 0x78, 0x0F, 0xF0, 0xFF, 0xA5,
    0x1E, 0x03, 0xFC, 0x3F, 0xE9, 0x47, 0x80, 0xFF, 0x0F, 0xFA, 0x51, 0xE0, 0x3F, 0xC3, 0xFE,
    0x94, 0x78, 0x0F, 0xF0, 0xFF, 0xB5, 0x1E, 0x03, 0xFC, 0x3F, 0xED, 0x47, 0x80, 0xFF, 0x0F,
    0xFB, 0x51, 0xE0, 0x3F, 0xC3, 0xFE, 0xD4, 0x78, 0x0F, 0xF0, 0xFF, 0xB5, 0x1E, 0x03, 0xFC,
    0x3F, 0xED, 0x47, 0x80, 0xFF, 0x0F, 0xFB, 0x51, 0xE0, 0x3F, 0xC3, 0xFE, 0xD4, 0x78, 0x0F,
    0xF0, 0xFF, 0xB5, 0x1E, 0x03, 0xFC, 0x3F, 0xED, 0x47, 0x80, 0xFF, 0x0F, 0xFA, 0x51, 0xE0,
    0x3F, 0xC3, 0xFE, 0x94, 0x78, 0x0F, 0xF0, 0xFF, 0xA5, 0x1E, 0x03, 0xFC, 0x3F, 0xE9, 0x47,
    0x80, 0xFF, 0x0F, 0xFA, 0x51, 0xE0, 0x3F, 0xC3, 0xFE, 0x94, 0x78, 0x0F, 0xF0, 0xFF, 0xA5,
    0x1E, 0x03, 0xFC, 0x3F, 0xE8, 0x47, 0x80, 0xFF, 0x0F, 0xFA, 0x11, 0xE0, 0x3F, 0xC3, 0xFE,
    0x84, 0x78, 0x0F, 0xF0, 0xFF, 0xA1, 0x1E, 0x03, 0xFC, 0x3F, 0xE8, 0x47, 0x80, 0xFF, 0x0F,
    0xFA, 0x11, 0xE0, 0x3F, 0xC3, 0xFE, 0x84, 0x78, 0x0F, 0xF0, 0xFF, 0xA1, 0x1E, 0x03, 0xFC,
    0x3F, 0xE8, 0x47, 0x80, 0xFF, 0x0F, 0xFA, 0x11, 0xE0, 0x3F, 0xC3, 0xFE, 0x84, 0x78, 0x0F,
    0xF0, 0xFF, 0xA1, 0x1E, 0x03, 0xFC, 0x3F, 0xE8, 0x47, 0x80, 0xFF, 0x0F, 0xFA, 0x11, 0xE0,
    0x3F, 0xC3, 0xFE, 0x84, 0x78, 0x0F, 0xF0, 0xFF, 0xA1, 0x1E, 0x03, 0xFC, 0x1F, 0xE8, 0x47,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};
const struct lux1310_wavetab lux1310_wt_sram80 = {
    .read_delay = 80,
    .start_delay = 25,
    .len = sizeof(lux1310_wt_sram80_data),
    .table = lux1310_wt_sram80_data
};

static const uint8_t lux1310_wt_sram39_blk_data[] = {
    0x81, 0xFF, 0x2F, 0xFA, 0x31, 0xE0, 0x7F, 0xCB, 0xFE, 0x8C, 0x78, 0x1F, 0xF2, 0xFF, 0xA3,
    0x1E, 0x07, 0xFC, 0xBF, 0xE8, 0xC7, 0x81, 0xFF, 0x2F, 0xFA, 0x31, 0xE0, 0x7F, 0xCB, 0xFE,
    0x8C, 0x78, 0x1F, 0xF2, 0xFF, 0xA3, 0x1E, 0x07, 0xFC, 0xBF, 0xE8, 0x47, 0x81, 0xFF, 0x2F,
    0xFA, 0x11, 0xE0, 0x7F, 0xCB, 0xFE, 0x84, 0x78, 0x1F, 0xF2, 0xFF, 0xA1, 0x1E, 0x07, 0xFC,
    0xBF, 0xE8, 0x47, 0x81, 0xFF, 0x2F, 0xFA, 0x11, 0xE0, 0x3F, 0xCB, 0xFE, 0x84, 0x78, 0x0F,
    0xF0, 0xFF, 0xB5, 0x1E, 0x03, 0xFC, 0x3F, 0xED, 0x47, 0x80, 0xFF, 0x0F, 0xFB, 0x51, 0xE0,
    0x3F, 0xC3, 0xFE, 0xD4, 0x78, 0x0F, 0xF0, 0xFF, 0xB5, 0x1E, 0x03, 0xFC, 0x3F, 0xED, 0x47,
    0x80, 0xFF, 0x0F, 0xFB, 0x51, 0xE0, 0x3F, 0xC3, 0xFE, 0xD4, 0x78, 0x0F, 0xF0, 0xFF, 0xB5,
    0x1E, 0x03, 0xFC, 0x3F, 0xE9, 0x47, 0x80, 0xFF, 0x0F, 0xFA, 0x51, 0xE0, 0x3F, 0xC3, 0xFE,
    0x94, 0x78, 0x0F, 0xF0, 0xFF, 0xA5, 0x1E, 0x03, 0xFC, 0x3F, 0xE8, 0x47, 0x80, 0xFF, 0x0F,
    0xFA, 0x11, 0xE0, 0x3F, 0xC3, 0xFE, 0x84, 0x78, 0x0F, 0xF0, 0xFF, 0xA1, 0x1E, 0x03, 0xFC,
    0x3F, 0xE8, 0x47, 0x80, 0xFF, 0x0F, 0xFA, 0x11, 0xE0, 0x3F, 0xC3, 0xFE, 0x84, 0x78, 0x0F,
    0xF0, 0xFF, 0xA1, 0x1E, 0x03, 0xFC, 0x3F, 0xE8, 0x47, 0x80, 0xFF, 0x0F, 0xFA, 0x11, 0xE8,
    0x3F, 0xC3, 0xFE, 0x84, 0x78, 0x0F, 0xF0, 0x7F, 0xA1, 0x1C, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00
};
const struct lux1310_wavetab lux1310_wt_sram39_blk = {
    .read_delay = 39,
    .start_delay = 30,
    .len = sizeof(lux1310_wt_sram39_blk_data),
    .table = lux1310_wt_sram39_blk_data
};

static const uint8_t lux1310_wt_sram39_data[] = {
    0x81, 0xFF, 0x2F, 0xFA, 0x31, 0xE0, 0x7F, 0xCB, 0xFE, 0x8C, 0x78, 0x1F, 0xF2, 0xFF, 0xA3,
    0x1E, 0x07, 0xFC, 0xBF, 0xE8, 0xC7, 0x81, 0xFF, 0x2F, 0xFA, 0x31, 0xE0, 0x7F, 0xCB, 0xFE,
    0x8C, 0x78, 0x1F, 0xF2, 0xFF, 0xA3, 0x1E, 0x07, 0xFC, 0xBF, 0xE8, 0x47, 0x81, 0xFF, 0x2F,
    0xFA, 0x11, 0xE0, 0x7F, 0xCB, 0xFE, 0x84, 0x78, 0x1F, 0xF2, 0xFF, 0xA1, 0x1E, 0x07, 0xFC,
    0xBF, 0xE8, 0x47, 0x81, 0xFF, 0x2F, 0xFA, 0x11, 0xE0, 0x3F, 0xCB, 0xFE, 0x84, 0x78, 0x0F,
    0xF0, 0xFF, 0xB5, 0x1E, 0x03, 0xFC, 0x3F, 0xED, 0x47, 0x80, 0xFF, 0x0F, 0xFB, 0x51, 0xE0,
    0x3F, 0xC3, 0xFE, 0xD4, 0x78, 0x0F, 0xF0, 0xFF, 0xB5, 0x1E, 0x03, 0xFC, 0x3F, 0xED, 0x47,
    0x80, 0xFF, 0x0F, 0xFB, 0x51, 0xE0, 0x3F, 0xC3, 0xFE, 0xD4, 0x78, 0x0F, 0xF0, 0xFF, 0xB5,
    0x1E, 0x03, 0xFC, 0x3F, 0xE9, 0x47, 0x80, 0xFF, 0x0F, 0xFA, 0x51, 0xE0, 0x3F, 0xC3, 0xFE,
    0x94, 0x78, 0x0F, 0xF0, 0xFF, 0xA5, 0x1E, 0x03, 0xFC, 0x3F, 0xE8, 0x47, 0x80, 0xFF, 0x0F,
    0xFA, 0x11, 0xE0, 0x3F, 0xC3, 0xFE, 0x84, 0x78, 0x0F, 0xF0, 0xFF, 0xA1, 0x1E, 0x03, 0xFC,
    0x3F, 0xE8, 0x47, 0x80, 0xFF, 0x0F, 0xFA, 0x11, 0xE0, 0x3F, 0xC3, 0xFE, 0x84, 0x78, 0x0F,
    0xF0, 0xFF, 0xA1, 0x1E, 0x03, 0xFC, 0x3F, 0xE8, 0x47, 0x80, 0xFF, 0x0F, 0xFA, 0x11, 0xE0,
    0x3F, 0xC3, 0xFE, 0x84, 0x78, 0x0F, 0xF0, 0x7F, 0xA1, 0x1C, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00
};
const struct lux1310_wavetab lux1310_wt_sram39 = {
    .read_delay = 39,
    .start_delay = 30,
    .len = sizeof(lux1310_wt_sram39_data),
    .table = lux1310_wt_sram39_data
};

static const uint8_t lux1310_wt_sram30_data[] = {
    0x81, 0xFF, 0x2F, 0xFA, 0x31, 0xE0, 0x7F, 0xCB, 0xFE, 0x8C, 0x78, 0x1F, 0xF2, 0xFF, 0xA3,
    0x1E, 0x07, 0xFC, 0xBF, 0xE8, 0xC7, 0x81, 0xFF, 0x2F, 0xFA, 0x31, 0xE0, 0x7F, 0xCB, 0xFE,
    0x8C, 0x78, 0x1F, 0xF2, 0xFF, 0xA3, 0x1E, 0x07, 0xFC, 0xBF, 0xE8, 0x47, 0x81, 0xFF, 0x2F,
    0xFA, 0x11, 0xE0, 0x7F, 0xCB, 0xFE, 0x84, 0x78, 0x1F, 0xF2, 0xFF, 0xA1, 0x1E, 0x07, 0xFC,
    0xBF, 0xE8, 0x47, 0x81, 0xFF, 0x2F, 0xFA, 0x11, 0xE0, 0x3F, 0xCB, 0xFE, 0x84, 0x78, 0x0F,
    0xF0, 0xFF, 0xB5, 0x1E, 0x03, 0xFC, 0x3F, 0xED, 0x47, 0x80, 0xFF, 0x0F, 0xFB, 0x51, 0xE0,
    0x3F, 0xC3, 0xFE, 0xD4, 0x78, 0x0F, 0xF0, 0xFF, 0xB5, 0x1E, 0x03, 0xFC, 0x3F, 0xED, 0x47,
    0x80, 0xFF, 0x0F, 0xFA, 0x51, 0xE0, 0x3F, 0xC3, 0xFE, 0x94, 0x78, 0x0F, 0xF0, 0xFF, 0xA1,
    0x1E, 0x03, 0xFC, 0x3F, 0xE8, 0x47, 0x80, 0xFF, 0x0F, 0xFA, 0x11, 0xE0, 0x3F, 0xC3, 0xFE,
    0x84, 0x78, 0x0F, 0xF0, 0xFF, 0xA1, 0x1E, 0x03, 0xFC, 0x3F, 0xE8, 0x47, 0x80, 0xFF, 0x0F,
    0xFA, 0x11, 0xE0, 0x3F, 0xC1, 0xFE, 0x84, 0x70, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};
const struct lux1310_wavetab lux1310_wt_sram30 = {
    .read_delay = 30,
    .start_delay = 29,
    .len = sizeof(lux1310_wt_sram30_data),
    .table = lux1310_wt_sram30_data
};

static const uint8_t lux1310_wt_sram25_data[] = {
    0x81, 0xFF, 0x2F, 0xFA, 0x31, 0xE0, 0x7F, 0xCB, 0xFE, 0x8C, 0x78, 0x1F, 0xF2, 0xFF, 0xA3,
    0x1E, 0x07, 0xFC, 0xBF, 0xE8, 0xC7, 0x81, 0xFF, 0x2F, 0xFA, 0x31, 0xE0, 0x7F, 0xCB, 0xFE,
    0x8C, 0x78, 0x1F, 0xF2, 0xFF, 0xA1, 0x1E, 0x07, 0xFC, 0xBF, 0xE8, 0x47, 0x81, 0xFF, 0x2F,
    0xFA, 0x11, 0xE0, 0x7F, 0xCB, 0xFE, 0x84, 0x78, 0x1F, 0xF2, 0xFF, 0xA1, 0x1E, 0x03, 0xFC,
    0xBF, 0xE8, 0x47, 0x80, 0xFF, 0x0F, 0xFB, 0x51, 0xE0, 0x3F, 0xC3, 0xFE, 0xD4, 0x78, 0x0F,
    0xF0, 0xFF, 0xB5, 0x1E, 0x03, 0xFC, 0x3F, 0xED, 0x47, 0x80, 0xFF, 0x0F, 0xFB, 0x51, 0xE0,
    0x3F, 0xC3, 0xFE, 0xD4, 0x78, 0x0F, 0xF0, 0xFF, 0xA5, 0x1E, 0x03, 0xFC, 0x3F, 0xE9, 0x47,
    0x80, 0xFF, 0x0F, 0xFA, 0x11, 0xE0, 0x3F, 0xC3, 0xFE, 0x84, 0x78, 0x0F, 0xF0, 0xFF, 0xA1,
    0x1E, 0x03, 0xFC, 0x3F, 0xE8, 0x47, 0x80, 0xFF, 0x07, 0xFA, 0x11, 0xC0, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
};
const struct lux1310_wavetab lux1310_wt_sram25 = {
    .read_delay = 25,
    .start_delay = 27,
    .len = sizeof(lux1310_wt_sram25_data),
    .table = lux1310_wt_sram25_data
};

static const uint8_t lux1310_wt_sram20_data[] = {
    0x81, 0xFF, 0x2F, 0xFA, 0x31, 0xE0, 0x7F, 0xCB, 0xFE, 0x8C, 0x78, 0x1F, 0xF2, 0xFF, 0xA3,
    0x1E, 0x07, 0xFC, 0xBF, 0xE8, 0xC7, 0x81, 0xFF, 0x2F, 0xFA, 0x31, 0xE0, 0x7F, 0xCB, 0xFE,
    0x84, 0x78, 0x1F, 0xF2, 0xFF, 0xA1, 0x1E, 0x07, 0xFC, 0xBF, 0xE8, 0x47, 0x81, 0xFF, 0x2F,
    0xFA, 0x11, 0xE0, 0x3F, 0xCB, 0xFE, 0x84, 0x78, 0x0F, 0xF0, 0xFF, 0xB5, 0x1E, 0x03, 0xFC,
    0x3F, 0xED, 0x47, 0x80, 0xFF, 0x0F, 0xFB, 0x51, 0xE0, 0x3F, 0xC3, 0xFE, 0xD4, 0x78, 0x0F,
    0xF0, 0xFF, 0xB5, 0x1E, 0x03, 0xFC, 0x3F, 0xE9, 0x47, 0x80, 0xFF, 0x0F, 0xFA, 0x51, 0xE0,
    0x3F, 0xC3, 0xFE, 0x84, 0x78, 0x0F, 0xF0, 0xFF, 0xA1, 0x1E, 0x03, 0xFC, 0x1F, 0xE8, 0x47,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00
};
const struct lux1310_wavetab lux1310_wt_sram20 = {
    .read_delay = 20,
    .start_delay = 25,
    .len = sizeof(lux1310_wt_sram20_data),
    .table = lux1310_wt_sram20_data
};
